@@ -238,6 +238,7 @@ public:
     bool shouldSkip(const QString &path) const;
     void skipSrc(bool isDir);
     void renameDirectory(QList<CopyInfo>::iterator it, const QUrl &newUrl);
+    QUrl finalDestUrl(const QUrl &src, const QUrl &dest) const;
 
     void slotStart();
     void slotEntries(KIO::Job *, const KIO::UDSEntryList &list);
@@ -298,6 +299,20 @@ QUrl CopyJob::destUrl() const
 void CopyJobPrivate::slotStart()
 {
     Q_Q(CopyJob);
+    if (m_mode == CopyJob::CopyMode::Move) {
+        Q_FOREACH (const QUrl &url, m_srcList) {
+            if (m_dest.scheme() == url.scheme() && m_dest.host() == url.host()) {
+                QString srcPath = url.path();
+                if (!srcPath.endsWith(QLatin1Char('/')))
+                    srcPath += QLatin1Char('/');
+                if (m_dest.path().startsWith(srcPath)) {
+                    q->setError(KIO::ERR_CANNOT_MOVE_INTO_ITSELF);
+                    q->emitResult();
+                    return;
+                }
+            }
+        }
+    }
     /**
        We call the functions directly instead of using signals.
        Calling a function via a signal takes approx. 65 times the time
@@ -680,6 +695,21 @@ void CopyJobPrivate::addCopyInfoFromUDSEntry(const UDSEntry &entry, const QUrl &
     }
 }
 
+// Adjust for kio_trash choosing its own dest url...
+QUrl CopyJobPrivate::finalDestUrl(const QUrl& src, const QUrl &dest) const
+{
+    Q_Q(const CopyJob);
+    if (dest.scheme() == QLatin1String("trash")) {
+        const QMap<QString, QString>& metaData = q->metaData();
+        QMap<QString, QString>::ConstIterator it = metaData.find("trashURL-" + src.path());
+        if (it != metaData.constEnd()) {
+            qDebug() << "finalDestUrl=" << it.value();
+            return QUrl(it.value());
+        }
+    }
+    return dest;
+}
+
 void CopyJobPrivate::skipSrc(bool isDir)
 {
     m_dest = m_globalDest;
@@ -994,7 +1024,7 @@ void CopyJobPrivate::slotResultCreatingDirs(KJob *job)
                 // Did the user choose to overwrite already?
                 const QString destDir = (*it).uDest.path();
                 if (shouldOverwriteDir(destDir)) {     // overwrite => just skip
-                    emit q->copyingDone(q, (*it).uSource, (*it).uDest, (*it).mtime, true /* directory */, false /* renamed */);
+                    emit q->copyingDone(q, (*it).uSource, finalDestUrl((*it).uSource, (*it).uDest), (*it).mtime, true /* directory */, false /* renamed */);
                     dirs.erase(it);   // Move on to next dir
                 } else {
                     if (m_bAutoRenameDirs) {
@@ -1031,7 +1061,7 @@ void CopyJobPrivate::slotResultCreatingDirs(KJob *job)
         }
     } else { // no error : remove from list, to move on to next dir
         //this is required for the undo feature
-        emit q->copyingDone(q, (*it).uSource, (*it).uDest, (*it).mtime, true, false);
+        emit q->copyingDone(q, (*it).uSource, finalDestUrl((*it).uSource, (*it).uDest), (*it).mtime, true, false);
         m_directoriesCopied.append(*it);
         dirs.erase(it);
     }
@@ -1118,14 +1148,14 @@ void CopyJobPrivate::slotResultConflictCreatingDirs(KJob *job)
         break;
     case Result_Overwrite:
         m_overwriteList.insert(existingDest);
-        emit q->copyingDone(q, (*it).uSource, (*it).uDest, (*it).mtime, true /* directory */, false /* renamed */);
+        emit q->copyingDone(q, (*it).uSource, finalDestUrl((*it).uSource, (*it).uDest), (*it).mtime, true /* directory */, false /* renamed */);
         // Move on to next dir
         dirs.erase(it);
         m_processedDirs++;
         break;
     case Result_OverwriteAll:
         m_bOverwriteAllDirs = true;
-        emit q->copyingDone(q, (*it).uSource, (*it).uDest, (*it).mtime, true /* directory */, false /* renamed */);
+        emit q->copyingDone(q, (*it).uSource, finalDestUrl((*it).uSource, (*it).uDest), (*it).mtime, true /* directory */, false /* renamed */);
         // Move on to next dir
         dirs.erase(it);
         m_processedDirs++;
@@ -1162,7 +1192,7 @@ void CopyJobPrivate::createNextDir()
         KIO::SimpleJob *newjob = KIO::mkdir(udir, -1);
         Scheduler::setJobPriority(newjob, 1);
         if (shouldOverwriteFile(udir.path())) { // if we are overwriting an existing file or symlink
-            newjob->addMetaData("overwrite", "true");
+            newjob->addMetaData(QStringLiteral("overwrite"), QStringLiteral("true"));
         }
 
         m_currentDestURL = udir;
@@ -1266,15 +1296,17 @@ void CopyJobPrivate::slotResultCopyingFiles(KJob *job)
             return; // Don't move to next file yet !
         }
 
+        const QUrl finalUrl = finalDestUrl((*it).uSource, (*it).uDest);
+
         if (m_bCurrentOperationIsLink) {
             QString target = (m_mode == CopyJob::Link ? (*it).uSource.path() : (*it).linkDest);
             //required for the undo feature
-            emit q->copyingLinkDone(q, (*it).uSource, target, (*it).uDest);
+            emit q->copyingLinkDone(q, (*it).uSource, target, finalUrl);
         } else {
             //required for the undo feature
-            emit q->copyingDone(q, (*it).uSource, (*it).uDest, (*it).mtime, false, false);
+            emit q->copyingDone(q, (*it).uSource, finalUrl, (*it).mtime, false, false);
             if (m_mode == CopyJob::Move) {
-                org::kde::KDirNotify::emitFileMoved((*it).uSource, (*it).uDest);
+                org::kde::KDirNotify::emitFileMoved((*it).uSource, finalUrl);
             }
             m_successSrcList.append((*it).uSource);
             if (m_freeSpace != (KIO::filesize_t) - 1 && (*it).size != (KIO::filesize_t) - 1) {
@@ -1462,18 +1494,18 @@ KIO::Job *CopyJobPrivate::linkNextFile(const QUrl &uSource, const QUrl &uDest, J
                 url.setPassword(QString());
                 config.writePathEntry("URL", url.toString());
                 config.writeEntry("Name", url.toString());
-                config.writeEntry("Type", QString::fromLatin1("Link"));
+                config.writeEntry("Type", QStringLiteral("Link"));
                 QString protocol = uSource.scheme();
                 if (protocol == QLatin1String("ftp")) {
-                    config.writeEntry("Icon", QString::fromLatin1("folder-remote"));
+                    config.writeEntry("Icon", QStringLiteral("folder-remote"));
                 } else if (protocol == QLatin1String("http")) {
-                    config.writeEntry("Icon", QString::fromLatin1("text-html"));
+                    config.writeEntry("Icon", QStringLiteral("text-html"));
                 } else if (protocol == QLatin1String("info")) {
-                    config.writeEntry("Icon", QString::fromLatin1("text-x-texinfo"));
+                    config.writeEntry("Icon", QStringLiteral("text-x-texinfo"));
                 } else if (protocol == QLatin1String("mailto")) { // sven:
-                    config.writeEntry("Icon", QString::fromLatin1("internet-mail"));    // added mailto: support
+                    config.writeEntry("Icon", QStringLiteral("internet-mail"));    // added mailto: support
                 } else {
-                    config.writeEntry("Icon", QString::fromLatin1("unknown"));
+                    config.writeEntry("Icon", QStringLiteral("unknown"));
                 }
                 config.sync();
                 files.erase(files.begin());   // done with this one, move on
@@ -1538,11 +1570,19 @@ void CopyJobPrivate::copyNextFile()
             bOverwrite = shouldOverwriteFile(destFile);
         }
 
+        // If source isn't local and target is local, we ignore the original permissions
+        // Otherwise, files downloaded from HTTP end up with -r--r--r--
+        const bool remoteSource = !KProtocolManager::supportsListing(uSource) || uSource.scheme() == QLatin1String("trash");
+        int permissions = (*it).permissions;
+        if (m_defaultPermissions || (remoteSource && uDest.isLocalFile())) {
+            permissions = -1;
+        }
+        const JobFlags flags = bOverwrite ? Overwrite : DefaultFlags;
+
         m_bCurrentOperationIsLink = false;
         KIO::Job *newjob = 0;
         if (m_mode == CopyJob::Link) {
             // User requested that a symlink be made
-            const JobFlags flags = bOverwrite ? Overwrite : DefaultFlags;
             newjob = linkNextFile(uSource, uDest, flags);
             if (!newjob) {
                 return;
@@ -1555,7 +1595,6 @@ void CopyJobPrivate::copyNextFile()
                    (uSource.password() == uDest.password()))
             // Copying a symlink - only on the same protocol/host/etc. (#5601, downloading an FTP file through its link),
         {
-            const JobFlags flags = bOverwrite ? Overwrite : DefaultFlags;
             KIO::SimpleJob *newJob = KIO::symlink((*it).linkDest, uDest, flags | HideProgressInfo /*no GUI*/);
             Scheduler::setJobPriority(newJob, 1);
             newjob = newJob;
@@ -1568,8 +1607,7 @@ void CopyJobPrivate::copyNextFile()
             m_bCurrentOperationIsLink = true;
             // NOTE: if we are moving stuff, the deletion of the source will be done in slotResultCopyingFiles
         } else if (m_mode == CopyJob::Move) { // Moving a file
-            JobFlags flags = bOverwrite ? Overwrite : DefaultFlags;
-            KIO::FileCopyJob *moveJob = KIO::file_move(uSource, uDest, (*it).permissions, flags | HideProgressInfo/*no GUI*/);
+            KIO::FileCopyJob *moveJob = KIO::file_move(uSource, uDest, permissions, flags | HideProgressInfo/*no GUI*/);
             moveJob->setSourceSize((*it).size);
             moveJob->setModificationTime((*it).mtime); // #55804
             newjob = moveJob;
@@ -1580,14 +1618,6 @@ void CopyJobPrivate::copyNextFile()
             m_bURLDirty = true;
             //Observer::self()->slotMoving( this, uSource, uDest );
         } else { // Copying a file
-            // If source isn't local and target is local, we ignore the original permissions
-            // Otherwise, files downloaded from HTTP end up with -r--r--r--
-            bool remoteSource = !KProtocolManager::supportsListing(uSource);
-            int permissions = (*it).permissions;
-            if (m_defaultPermissions || (remoteSource && uDest.isLocalFile())) {
-                permissions = -1;
-            }
-            JobFlags flags = bOverwrite ? Overwrite : DefaultFlags;
             KIO::FileCopyJob *copyJob = KIO::file_copy(uSource, uDest, permissions, flags | HideProgressInfo/*no GUI*/);
             copyJob->setParentJob(q);   // in case of rename dialog
             copyJob->setSourceSize((*it).size);
@@ -1807,15 +1837,15 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
                     tmpFile.close();
                     tmpFile.remove();
                     //qDebug() << "QTemporaryFile using" << _tmp << "as intermediary";
-                    if (QFile::rename(_src, _tmp) == 0) {
+                    if (QFile::rename(_src, _tmp)) {
                         //qDebug() << "Renaming" << _src << "to" << _tmp << "succeeded";
-                        if (!QFile::exists(_dest) && QFile::rename(_tmp, _dest) == 0) {
+                        if (!QFile::exists(_dest) && QFile::rename(_tmp, _dest)) {
                             err = 0;
                             org::kde::KDirNotify::emitFileRenamed(m_currentSrcURL, dest);
                         } else {
                             //qDebug() << "Didn't manage to rename" << _tmp << "to" << _dest << ", reverting";
                             // Revert back to original name!
-                            if (QFile::rename(_tmp, _src) != 0) {
+                            if (!QFile::rename(_tmp, _src)) {
                                 qWarning() << "Couldn't rename" << _tmp << "back to" << _src << '!';
                                 // Severe error, abort
                                 q->Job::slotResult(job); // will set the error and emit result(this)
@@ -2005,7 +2035,7 @@ void CopyJobPrivate::slotResultRenaming(KJob *job)
     } else {
         //qDebug() << "Renaming succeeded, move on";
         ++m_processedFiles;
-        emit q->copyingDone(q, *m_currentStatSrc, dest, QDateTime() /*mtime unknown, and not needed*/, true, true);
+        emit q->copyingDone(q, *m_currentStatSrc, finalDestUrl(*m_currentStatSrc, dest), QDateTime() /*mtime unknown, and not needed*/, true, true);
         m_successSrcList.append(*m_currentStatSrc);
         statNextSrc();
     }
@@ -2170,12 +2200,12 @@ CopyJob *KIO::trash(const QUrl &src, JobFlags flags)
 {
     QList<QUrl> srcList;
     srcList.append(src);
-    return CopyJobPrivate::newJob(srcList, QUrl("trash:/"), CopyJob::Move, false, flags);
+    return CopyJobPrivate::newJob(srcList, QUrl(QStringLiteral("trash:/")), CopyJob::Move, false, flags);
 }
 
 CopyJob *KIO::trash(const QList<QUrl> &srcList, JobFlags flags)
 {
-    return CopyJobPrivate::newJob(srcList, QUrl("trash:/"), CopyJob::Move, false, flags);
+    return CopyJobPrivate::newJob(srcList, QUrl(QStringLiteral("trash:/")), CopyJob::Move, false, flags);
 }
 
 #include "moc_copyjob.cpp"
